@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: nm-vpn-properties.c,v 1.7 2005/10/07 19:17:05 rml Exp $
+ * CVSID: $Id: nm-vpn-properties.c 2115 2006-11-20 04:07:05Z dcbw $
  *
  * nm-vpn-properties.c : GNOME UI dialogs for manipulating VPN connections
  *
@@ -39,16 +39,16 @@
 #include <gtk/gtkwindow.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
-#include <glib/gi18n-lib.h>
+#include <glib/gi18n.h>
 
 #define NM_VPN_API_SUBJECT_TO_CHANGE
 #include "nm-vpn-ui-interface.h"
+#include "clipboard.h"
 
 #define NM_GCONF_VPN_CONNECTIONS_PATH "/system/networking/vpn_connections"
 
 static GladeXML *xml;
 static GConfClient *gconf_client;
-
 static GtkWidget *dialog;
 static GtkWindow *druid_window;
 static GtkTreeView *vpn_conn_view;
@@ -61,7 +61,6 @@ static GnomeDruidPageEdge *druid_confirm_page;
 static GtkComboBox *vpn_type_combo_box;
 static GtkVBox *vpn_type_details;
 static GtkDialog *edit_dialog;
-
 static GSList *vpn_types;
 
 static NetworkManagerVpnUI *
@@ -69,11 +68,15 @@ find_vpn_ui_by_service_name (const char *service_name)
 {
 	GSList *i;
 
+	g_return_val_if_fail (service_name != NULL, NULL);
+
 	for (i = vpn_types; i != NULL; i = g_slist_next (i)) {
 		NetworkManagerVpnUI *vpn_ui;
+		const char * vpn_ui_service_name;
 
 		vpn_ui = i->data;
-		if (strcmp (vpn_ui->get_service_name (vpn_ui), service_name) == 0)
+		vpn_ui_service_name = vpn_ui->get_service_name (vpn_ui);
+		if (vpn_ui_service_name && strcmp (vpn_ui_service_name, service_name) == 0)
 			return vpn_ui;
 	}
 
@@ -91,35 +94,35 @@ enum {
 static void
 update_edit_del_sensitivity (void)
 {
-	GtkTreeIter iter;
 	GtkTreeSelection *selection;
-	gboolean is_editable;	
-	char *service_name;
-	NetworkManagerVpnUI *vpn_ui;
+	gboolean is_editable = FALSE, is_exportable = FALSE;
+	GtkTreeIter iter;
 
-	if ((selection = gtk_tree_view_get_selection (vpn_conn_view)) == NULL)
-		goto out;
+	selection = gtk_tree_view_get_selection (vpn_conn_view);
+	if (!selection || !gtk_tree_selection_get_selected (selection, NULL, &iter))
+		is_editable = is_exportable = FALSE;
+	else {
+		NetworkManagerVpnUI *vpn_ui;
+		const char *service_name = NULL;
 
-	if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
-		goto out;
+		gtk_tree_model_get (GTK_TREE_MODEL (vpn_conn_list), &iter, VPNCONN_USER_CAN_EDIT_COLUMN, &is_editable, -1);
+		gtk_tree_model_get (GTK_TREE_MODEL (vpn_conn_list), &iter, VPNCONN_SVC_NAME_COLUMN, &service_name, -1);
 
-	gtk_tree_model_get (GTK_TREE_MODEL (vpn_conn_list), &iter, VPNCONN_USER_CAN_EDIT_COLUMN, &is_editable, -1);
-	gtk_tree_model_get (GTK_TREE_MODEL (vpn_conn_list), &iter, VPNCONN_SVC_NAME_COLUMN, &service_name, -1);
+		vpn_ui = find_vpn_ui_by_service_name (service_name);
+		if (vpn_ui)
+			is_exportable = vpn_ui->can_export (vpn_ui);
+	}
 
 	gtk_widget_set_sensitive (vpn_edit, is_editable);
 	gtk_widget_set_sensitive (vpn_delete, is_editable);
-
-	vpn_ui = find_vpn_ui_by_service_name (service_name);
-	gtk_widget_set_sensitive (vpn_export, vpn_ui->can_export (vpn_ui));
-
-out:
-	;
+	gtk_widget_set_sensitive (vpn_export, is_editable && is_exportable);
 }
 
 static gboolean
 add_vpn_connection (const char *conn_name, const char *service_name, GSList *conn_data, GSList *routes)
 {
 	char *gconf_key;
+	GtkTreeSelection *selection;
 	GtkTreeIter iter;
 	char conn_gconf_path[PATH_MAX];
 	char *escaped_conn_name;
@@ -137,7 +140,10 @@ add_vpn_connection (const char *conn_name, const char *service_name, GSList *con
 
 	if (gconf_client_dir_exists (gconf_client, conn_gconf_path, NULL))
 		goto out;
-	       
+
+	if ((selection = gtk_tree_view_get_selection (vpn_conn_view)) == NULL)
+		goto out;
+
 	/* User-visible name of connection */
 	gconf_key = g_strdup_printf ("%s/name", conn_gconf_path);
 	gconf_client_set_string (gconf_client, gconf_key, conn_name, NULL);
@@ -154,16 +160,19 @@ add_vpn_connection (const char *conn_name, const char *service_name, GSList *con
 
 	/* routes */
 	gconf_key = g_strdup_printf ("%s/routes", conn_gconf_path);
+
+#if 0
 	{
-/*
 		GSList *i;
 
 		i = NULL;
 		i = g_slist_append (i, "172.16.0.0/16");
-*/
 		gconf_client_set_list (gconf_client, gconf_key, GCONF_VALUE_STRING, routes, NULL);
-		/*g_slist_free (i);*/
+		g_slist_free (i);
 	}
+#else
+		gconf_client_set_list (gconf_client, gconf_key, GCONF_VALUE_STRING, routes, NULL);
+#endif
 
 	gconf_client_suggest_sync (gconf_client, NULL);
 
@@ -176,6 +185,7 @@ add_vpn_connection (const char *conn_name, const char *service_name, GSList *con
 			    VPNCONN_GCONF_COLUMN, conn_gconf_path,
 			    VPNCONN_USER_CAN_EDIT_COLUMN, &conn_user_can_edit,
 			    -1);
+	gtk_tree_selection_select_iter (selection, &iter);
 
 	ret = TRUE;
 
@@ -184,6 +194,36 @@ out:
 	return ret;
 }
 
+static void
+remove_vpn_connection (const char *gconf_path, GtkTreeIter *iter)
+{
+	char key[PATH_MAX];
+
+	g_snprintf (key, sizeof (key), "%s/name", gconf_path);
+	gconf_client_unset (gconf_client, key, NULL);
+	g_snprintf (key, sizeof (key), "%s/service_name", gconf_path);
+	gconf_client_unset (gconf_client, key, NULL);
+	g_snprintf (key, sizeof (key), "%s/vpn_data", gconf_path);
+	gconf_client_unset (gconf_client, key, NULL);
+	g_snprintf (key, sizeof (key), "%s/routes", gconf_path);
+	gconf_client_unset (gconf_client, key, NULL);
+	g_snprintf (key, sizeof (key), "%s/user_name", gconf_path);
+	gconf_client_unset (gconf_client, key, NULL);
+
+	g_snprintf (key, sizeof (key), "%s/last_attempt_success", gconf_path);
+	gconf_client_unset (gconf_client, key, NULL);
+
+	gconf_client_unset (gconf_client, gconf_path, NULL);
+	gconf_client_suggest_sync (gconf_client, NULL);
+
+	if (gtk_list_store_remove (vpn_conn_list, iter)) {
+		GtkTreeSelection *selection;
+
+		selection = gtk_tree_view_get_selection (vpn_conn_view);
+		gtk_tree_selection_select_iter (selection, iter);
+	}
+}
+
 static void 
 vpn_druid_vpn_validity_changed (NetworkManagerVpnUI *vpn_ui,
 				gboolean is_valid, 
@@ -192,8 +232,6 @@ vpn_druid_vpn_validity_changed (NetworkManagerVpnUI *vpn_ui,
 	char *conn_name;
 	GtkTreeIter iter;
 
-	/*printf ("vpn_druid_vpn_validity_changed %d!\n", is_valid);*/
-
 	conn_name = vpn_ui->get_connection_name (vpn_ui);
 
 	/* get list of existing connection names */
@@ -208,7 +246,6 @@ vpn_druid_vpn_validity_changed (NetworkManagerVpnUI *vpn_ui,
 					    -1);
 			
 			if (strcmp (name, conn_name) == 0) {
-				/*printf ("name '%s' is already in use\n", conn_name);*/
 				is_valid = FALSE;
 				break;
 			}
@@ -233,13 +270,14 @@ static gboolean vpn_druid_vpn_type_page_next (GnomeDruidPage *druidpage,
 	GtkWidget *w;
 	NetworkManagerVpnUI *vpn_ui;
 
-	/*printf ("vpn_type_next!\n");*/
-
 	/* first hide existing child */
 	w = g_list_nth_data (gtk_container_children (GTK_CONTAINER (vpn_type_details)), 0);
-	if (w != NULL) {
+	if (w)
+		gtk_widget_hide (w);
+	/* hide the previous widget from the next page, in case the user hit 'back' */
+	w = g_list_nth_data (gtk_container_children (GTK_CONTAINER (vpn_type_details)), 1);
+	if (w)
 		gtk_widget_hide (w);
-	}
 
 	/* show appropriate child */
 	vpn_ui = (NetworkManagerVpnUI *) g_slist_nth_data (vpn_types, gtk_combo_box_get_active (vpn_type_combo_box));
@@ -267,14 +305,17 @@ static void vpn_druid_vpn_details_page_prepare (GnomeDruidPage *druidpage,
 						GtkWidget *widget,
 						gpointer user_data)
 {
-	/*printf ("vpn_details_prepare!\n");*/
+	gboolean is_valid;
+	NetworkManagerVpnUI *vpn_ui;
 
-	gnome_druid_set_buttons_sensitive (druid, 
-					   TRUE,
-					   FALSE,
-					   TRUE,
-					   FALSE);	
+	is_valid = FALSE;
 
+	/* validate input, in case we are coming in via 'Back' */
+	vpn_ui = (NetworkManagerVpnUI *) g_slist_nth_data (vpn_types, gtk_combo_box_get_active (vpn_type_combo_box));
+	if (vpn_ui != NULL)
+		is_valid = vpn_ui->is_valid (vpn_ui);
+
+	gnome_druid_set_buttons_sensitive (druid, TRUE, is_valid, TRUE, FALSE);	
 }
 
 static gboolean vpn_druid_vpn_details_page_next (GnomeDruidPage *druidpage,
@@ -286,13 +327,10 @@ static gboolean vpn_druid_vpn_details_page_next (GnomeDruidPage *druidpage,
 
 	is_valid = FALSE;
 
-	/*printf ("vpn_details_next!\n");*/
-
 	/* validate input */
 	vpn_ui = (NetworkManagerVpnUI *) g_slist_nth_data (vpn_types, gtk_combo_box_get_active (vpn_type_combo_box));
-	if (vpn_ui != NULL) {
+	if (vpn_ui != NULL)
 		is_valid = vpn_ui->is_valid (vpn_ui);
-	}
 
 	return !is_valid;
 }
@@ -303,8 +341,6 @@ static void vpn_druid_vpn_confirm_page_prepare (GnomeDruidPage *druidpage,
 {
 	NetworkManagerVpnUI *vpn_ui;
 
-	/*printf ("vpn_confirm_prepare!\n");*/
-
 	vpn_ui = (NetworkManagerVpnUI *) g_slist_nth_data (vpn_types, gtk_combo_box_get_active (vpn_type_combo_box));
 	if (vpn_ui != NULL) {
 		gchar *confirm_text;
@@ -327,8 +363,6 @@ static gboolean vpn_druid_vpn_confirm_page_finish (GnomeDruidPage *druidpage,
 	char *conn_name;
 	NetworkManagerVpnUI *vpn_ui;
 
-	/*printf ("vpn_confirm_finish!\n");*/
-
 	vpn_ui = (NetworkManagerVpnUI *) g_slist_nth_data (vpn_types, gtk_combo_box_get_active (vpn_type_combo_box));
 	conn_name   = vpn_ui->get_connection_name (vpn_ui);
 	conn_data   = vpn_ui->get_properties (vpn_ui);
@@ -341,16 +375,17 @@ static gboolean vpn_druid_vpn_confirm_page_finish (GnomeDruidPage *druidpage,
 	return FALSE;
 }
 
-static gboolean vpn_druid_cancel (GnomeDruid *druid,
-				  gpointer user_data)
+static gboolean vpn_druid_cancel (GnomeDruid *ignored_druid, gpointer user_data)
 {
-	/*printf ("vpn_druid_cancel!\n");*/
-
 	gtk_widget_hide_all (GTK_WIDGET (druid_window));
 	return FALSE;
 }
 
-
+static gboolean vpn_window_close (GtkWidget *ignored, gpointer user_data)
+{
+	gtk_widget_hide_all (GTK_WIDGET (druid_window));
+	return TRUE;
+}
 
 static void
 add_cb (GtkButton *button, gpointer user_data)
@@ -359,25 +394,22 @@ add_cb (GtkButton *button, gpointer user_data)
 	GList *i;
 	GList *children;
 
-	/*printf ("add_cb\n");*/
-
 	/* Bail out if we don't have any VPN implementations on our system */
 	if (vpn_types == NULL || g_slist_length (vpn_types) == 0) {
-		GtkWidget *dialog;
+		GtkWidget *err_dialog;
 
-		dialog = gtk_message_dialog_new (NULL,
+		err_dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
 						 _("Cannot add VPN connection"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
 		   _("No suitable VPN software was found on your system. Contact your system administrator."));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+		gtk_dialog_run (GTK_DIALOG (err_dialog));
+		gtk_widget_destroy (err_dialog);
 		goto out;
 	}
 
-
 	/* remove existing VPN widget */
 	children = gtk_container_get_children (GTK_CONTAINER (vpn_type_details));
 	for (i = children; i != NULL; i = g_list_next (i)) {
@@ -395,6 +427,7 @@ add_cb (GtkButton *button, gpointer user_data)
 	gtk_window_set_policy (druid_window, FALSE, FALSE, TRUE);
 
 	gtk_widget_show_all (GTK_WIDGET (druid_window));
+
 out:
 	;
 }
@@ -408,27 +441,25 @@ import_settings (const char *svc_name, const char *name)
 	GList *children;
 	NetworkManagerVpnUI *vpn_ui;
 
-	/*printf ("import_settings svc_name='%s', name='%s' vpn-ui-=\n", svc_name, name);*/
-
 	vpn_ui = find_vpn_ui_by_service_name (svc_name);
 
 	/* Bail out if we don't have the requested VPN implementation on our system */
 	if (vpn_ui == NULL) {
 		char *basename;
-		GtkWidget *dialog;
+		GtkWidget *err_dialog;
 
 		basename = g_path_get_basename (name);
 
-		dialog = gtk_message_dialog_new (NULL,
+		err_dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
 						 _("Cannot import VPN connection"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
 							  _("Cannot find suitable software for VPN connection type '%s' to import the file '%s'. Contact your system administrator."),
 							  svc_name, basename);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+		gtk_dialog_run (GTK_DIALOG (err_dialog));
+		gtk_widget_destroy (err_dialog);
 		g_free (basename);
 		goto out;
 	}
@@ -445,7 +476,6 @@ import_settings (const char *svc_name, const char *name)
 	w = glade_xml_get_widget (xml, "vpn-druid-vpn-details-page");
 	gnome_druid_set_page (druid, GNOME_DRUID_PAGE (w));
 
-
 	/* show appropriate child */
 	w = vpn_ui->get_widget (vpn_ui, NULL, NULL, NULL);
 	if (w != NULL) {	
@@ -456,9 +486,9 @@ import_settings (const char *svc_name, const char *name)
 			gtk_container_remove (GTK_CONTAINER (old_parent), w);
 		gtk_container_add (GTK_CONTAINER (vpn_type_details), w);
 		gtk_widget_unref (w);
-		
 		gtk_widget_show_all (w);
-	}	
+	}
+
 	vpn_ui->set_validity_changed_callback (vpn_ui, vpn_druid_vpn_validity_changed, NULL);
 
 	vpn_ui->import_file (vpn_ui, name);
@@ -467,6 +497,7 @@ import_settings (const char *svc_name, const char *name)
 
 	gtk_window_set_policy (druid_window, FALSE, FALSE, TRUE);
 	gtk_widget_show_all (GTK_WIDGET (druid_window));
+
 out:
 	;
 }
@@ -483,8 +514,6 @@ vpn_edit_vpn_validity_changed (NetworkManagerVpnUI *vpn_ui,
 
 	orig_conn_name = (const char *) user_data;
 
-	/*printf ("vpn_edit_vpn_validity_changed %d!\n", is_valid);*/
-
 	conn_name = vpn_ui->get_connection_name (vpn_ui);
 
 	/* get list of existing connection names */
@@ -501,7 +530,6 @@ vpn_edit_vpn_validity_changed (NetworkManagerVpnUI *vpn_ui,
 			/* Can override the original name (stored in user_data, see edit_cb()) */
 			if (strcmp (name, orig_conn_name) != 0) {			
 				if (strcmp (name, conn_name) == 0) {
-					/*printf ("name '%s' is already in use\n", conn_name);*/
 					is_valid = FALSE;
 					break;
 				}
@@ -559,19 +587,19 @@ retrieve_data_from_selected_connection (NetworkManagerVpnUI **vpn_ui,
 
 	*vpn_ui = find_vpn_ui_by_service_name (conn_service_name);
 	if (*vpn_ui == NULL) {
-		GtkWidget *dialog;
+		GtkWidget *err_dialog;
 
-		dialog = gtk_message_dialog_new (NULL,
+		err_dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
 						 _("Error retrieving VPN connection '%s'"),
 						 *conn_name);
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-  _("Could not find the UI files for VPN connection type '%s'. Contact your system administrator."),
-							  conn_service_name);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
+		    _("Could not find the UI files for VPN connection type '%s'. Contact your system administrator."),
+		    conn_service_name);
+		gtk_dialog_run (GTK_DIALOG (err_dialog));
+		gtk_widget_destroy (err_dialog);
 		goto out;
 	}
 
@@ -583,9 +611,9 @@ retrieve_data_from_selected_connection (NetworkManagerVpnUI **vpn_ui,
 
 	*conn_vpn_data = NULL;
 	for (i = conn_vpn_data_gconfvalue; i != NULL; i = g_slist_next (i)) {
-		const char *value;
-		value = gconf_value_get_string ((GConfValue *) i->data);
-		*conn_vpn_data = g_slist_append (*conn_vpn_data, (gpointer) value);
+		const char *val;
+		val = gconf_value_get_string ((GConfValue *) i->data);
+		*conn_vpn_data = g_slist_append (*conn_vpn_data, (gpointer) val);
 	}
 
 
@@ -598,9 +626,9 @@ retrieve_data_from_selected_connection (NetworkManagerVpnUI **vpn_ui,
 	conn_routes_gconfvalue = gconf_value_get_list (value);
 	*conn_routes = NULL;
 	for (i = conn_routes_gconfvalue; i != NULL; i = g_slist_next (i)) {
-		const char *value;
-		value = gconf_value_get_string ((GConfValue *) i->data);
-		*conn_routes = g_slist_append (*conn_routes, (gpointer) value);
+		const char *val;
+		val = gconf_value_get_string ((GConfValue *) i->data);
+		*conn_routes = g_slist_append (*conn_routes, (gpointer) val);
 	}
 
 	result = TRUE;
@@ -620,13 +648,18 @@ edit_cb (GtkButton *button, gpointer user_data)
 	const char *conn_name;
 	char key[PATH_MAX];
 	char *conn_gconf_path;
+	GtkTreeSelection *selection;
 	GtkTreeIter iter;
 
-	/*printf ("edit\n");*/
-
 	if (!retrieve_data_from_selected_connection (&vpn_ui, &conn_vpn_data, &conn_routes, &conn_name, &conn_gconf_path))
 		goto out;
 
+	if ((selection = gtk_tree_view_get_selection (vpn_conn_view)) == NULL)
+		goto out;
+
+	if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+		goto out;
+
 	vpn_edit_widget = vpn_ui->get_widget (vpn_ui, conn_vpn_data, conn_routes, conn_name);
 
 	g_slist_free (conn_vpn_data);
@@ -656,7 +689,6 @@ edit_cb (GtkButton *button, gpointer user_data)
 		new_conn_routes = vpn_ui->get_routes (vpn_ui);
 
 		if (strcmp (new_conn_name, conn_name) == 0) {
-
 			/* same name, just update properties and routes */
 			g_snprintf (key, sizeof (key), "%s/vpn_data", conn_gconf_path);
 			gconf_client_set_list (gconf_client, key, GCONF_VALUE_STRING, new_conn_data, NULL);
@@ -665,26 +697,12 @@ edit_cb (GtkButton *button, gpointer user_data)
 
 			gconf_client_suggest_sync (gconf_client, NULL);
 		} else {
+			selection = gtk_tree_view_get_selection (vpn_conn_view);
+			gtk_tree_selection_get_selected (selection, NULL, &iter);
+			remove_vpn_connection (conn_gconf_path, &iter);
 
-			/* remove old entry */
-			g_snprintf (key, sizeof (key), "%s/name", conn_gconf_path);
-			gconf_client_unset (gconf_client, key, NULL);
-			g_snprintf (key, sizeof (key), "%s/service_name", conn_gconf_path);
-			gconf_client_unset (gconf_client, key, NULL);
-			g_snprintf (key, sizeof (key), "%s/vpn_data", conn_gconf_path);
-			gconf_client_unset (gconf_client, key, NULL);
-			/* TODO: at some point remove routes and user_name */
-			g_snprintf (key, sizeof (key), "%s/routes", conn_gconf_path);
-			gconf_client_unset (gconf_client, key, NULL);
-			g_snprintf (key, sizeof (key), "%s/user_name", conn_gconf_path);
-			gconf_client_unset (gconf_client, key, NULL);
-			gconf_client_unset (gconf_client, conn_gconf_path, NULL);
-			gconf_client_suggest_sync (gconf_client, NULL);
-			gtk_list_store_remove (vpn_conn_list, &iter);
-
-			/* add new entry */
 			add_vpn_connection (new_conn_name, vpn_ui->get_service_name (vpn_ui), 
-					    new_conn_data, new_conn_routes);
+								new_conn_data, new_conn_routes);
 		}
 
 		if (new_conn_data != NULL) {
@@ -695,10 +713,9 @@ edit_cb (GtkButton *button, gpointer user_data)
 			g_slist_foreach (new_conn_routes, (GFunc)g_free, NULL);
 			g_slist_free (new_conn_routes);
 		}
-
-
 	}
 
+	gtk_widget_hide (GTK_WIDGET (vpn_edit_widget));
 	gtk_widget_hide (GTK_WIDGET (edit_dialog));
 
 out:
@@ -712,11 +729,9 @@ delete_cb (GtkButton *button, gpointer user_data)
 	GtkTreeSelection *selection;
 	gchar *conn_gconf_path;
 	gchar *conn_name;
-	GtkWidget *dialog;
+	GtkWidget *confirm_dialog;
 	int response;
 
-	/*printf ("delete\n");*/
-
 	if ((selection = gtk_tree_view_get_selection (vpn_conn_view)) == NULL)
 		goto out;
 
@@ -724,43 +739,24 @@ delete_cb (GtkButton *button, gpointer user_data)
 		goto out;
 
 	gtk_tree_model_get (GTK_TREE_MODEL (vpn_conn_list), &iter, VPNCONN_NAME_COLUMN, &conn_name, -1);
-	dialog = gtk_message_dialog_new (NULL,
+	confirm_dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_WARNING,
 					 GTK_BUTTONS_CANCEL,
 					 _("Delete VPN connection \"%s\"?"), conn_name);
-	gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_DELETE, GTK_RESPONSE_OK, NULL);
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+	gtk_dialog_add_buttons (GTK_DIALOG (confirm_dialog), GTK_STOCK_DELETE, GTK_RESPONSE_OK, NULL);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (confirm_dialog),
 						  _("All information about the VPN connection \"%s\" will be lost and you may need your system administrator to provide information to create a new connection."), conn_name);
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	response = gtk_dialog_run (GTK_DIALOG (confirm_dialog));
+	gtk_widget_destroy (confirm_dialog);
 
 	if (response != GTK_RESPONSE_OK)
 		goto out;
 
 	gtk_tree_model_get (GTK_TREE_MODEL (vpn_conn_list), &iter, VPNCONN_GCONF_COLUMN, &conn_gconf_path, -1);
 
-	if (conn_gconf_path != NULL) {
-		char key[PATH_MAX];
-		
-		g_snprintf (key, sizeof (key), "%s/name", conn_gconf_path);
-		gconf_client_unset (gconf_client, key, NULL);
-		g_snprintf (key, sizeof (key), "%s/service_name", conn_gconf_path);
-		gconf_client_unset (gconf_client, key, NULL);
-		g_snprintf (key, sizeof (key), "%s/vpn_data", conn_gconf_path);
-		gconf_client_unset (gconf_client, key, NULL);
-		g_snprintf (key, sizeof (key), "%s/routes", conn_gconf_path);
-		gconf_client_unset (gconf_client, key, NULL);
-		/* TODO: remove user_name */
-		g_snprintf (key, sizeof (key), "%s/user_name", conn_gconf_path);
-		gconf_client_unset (gconf_client, key, NULL);
-		gconf_client_unset (gconf_client, conn_gconf_path, NULL);
-
-		gconf_client_suggest_sync (gconf_client, NULL);
-
-		if (gtk_list_store_remove (vpn_conn_list, &iter))
-			gtk_tree_selection_select_iter (selection, &iter);
-	}
+	if (conn_gconf_path != NULL)
+		remove_vpn_connection (conn_gconf_path, &iter);
 
 	update_edit_del_sensitivity ();
 
@@ -769,12 +765,19 @@ out:
 }
 
 static void
-close_cb (void)
+response_cb (void)
 {
 	gtk_widget_destroy (dialog);
 	gtk_main_quit ();
 }
 
+static gboolean
+delete_event_cb (GtkDialog *the_dialog)
+{
+	gtk_dialog_response (the_dialog, GTK_RESPONSE_DELETE_EVENT);
+	return TRUE;
+}
+
 static void
 export_cb (GtkButton *button, gpointer user_data)
 {
@@ -784,8 +787,6 @@ export_cb (GtkButton *button, gpointer user_data)
 	const char *conn_name;
 	char *conn_gconf_path;
 
-	/*printf ("edit\n");*/
-
 	if (!retrieve_data_from_selected_connection (&vpn_ui, &conn_vpn_data, &conn_routes, &conn_name, &conn_gconf_path))
 		goto out;
 
@@ -829,7 +830,7 @@ static void get_all_vpn_connections (void)
 		    gconf_value_get_list_type (value) != GCONF_VALUE_STRING ||
 		    (conn_vpn_data = gconf_value_get_list (value)) == NULL)
 			goto error;
-		
+
 		//conn_user_can_edit = (strcmp (conn_name, "RH VPN Boston") != 0);
 
 		gtk_list_store_append (vpn_conn_list, &iter);
@@ -840,7 +841,7 @@ static void get_all_vpn_connections (void)
 				    VPNCONN_USER_CAN_EDIT_COLUMN, conn_user_can_edit,
 				    -1);
 
-		/*
+#if 0
 		printf ("conn_name = '%s'\n", conn_name);
 		printf ("conn_service_name = '%s'\n", conn_service_name);
 		printf ("conn_vpn_data = {");
@@ -853,7 +854,7 @@ static void get_all_vpn_connections (void)
 			}
 			printf ("}\n");
 		}
-		*/
+#endif
 
 error:
 		g_free (vpn_conn->data);
@@ -864,8 +865,6 @@ static void
 vpn_list_cursor_changed_cb (GtkTreeView *treeview,
 			    gpointer user_data)
 {
-	/*printf ("*** vpn_list_cursor_changed_cb\n");*/
-
 	update_edit_del_sensitivity ();
 }
 
@@ -874,7 +873,7 @@ vpn_list_cursor_changed_cb (GtkTreeView *treeview,
 extern NetworkManagerVpnUI* vpn_ui_factory_dummy (void);
 
 static void
-load_properties_module (GSList **vpn_types, const char *path)
+load_properties_module (GSList **vpn_types_list, const char *path)
 {
 	GModule *module;
 	NetworkManagerVpnUI* (*nm_vpn_properties_factory) (void) = NULL;
@@ -888,7 +887,6 @@ load_properties_module (GSList **vpn_types, const char *path)
 
 	if (!g_module_symbol (module, "nm_vpn_properties_factory", 
 			      (gpointer) &nm_vpn_properties_factory)) {
-		
 		g_warning ("Cannot locate function 'nm_vpn_properties_factory' in '%s': %s", 
 			   path, g_module_error ());
 		g_module_close (module);
@@ -902,13 +900,36 @@ load_properties_module (GSList **vpn_types, const char *path)
 		goto out;
 	}
 
-	*vpn_types = g_slist_append (*vpn_types, impl);
+	*vpn_types_list = g_slist_append (*vpn_types_list, impl);
 
 out:
 	;
 }
 
-#define VPN_NAME_FILES_DIR "/etc/NetworkManager/VPN"
+#define VPN_NAME_FILES_DIR SYSCONFDIR"/NetworkManager/VPN"
+
+static gint
+vpn_list_sorter(GtkTreeModel *model,
+                GtkTreeIter *a,
+                GtkTreeIter *b,
+                gpointer user_data)
+{
+	GValue aval = {0};
+	GValue bval = {0};
+	const char *aname;
+	const char *bname;
+	gint res;
+
+	gtk_tree_model_get_value(model, a, VPNCONN_NAME_COLUMN, &aval);
+	gtk_tree_model_get_value(model, b, VPNCONN_NAME_COLUMN, &bval);
+	aname = g_value_get_string(&aval);
+	bname = g_value_get_string(&bval);
+	res = strcasecmp(aname, bname);
+	g_value_unset(&aval);
+	g_value_unset(&bval);
+	return res;
+}
+
 
 static gboolean
 init_app (void)
@@ -919,9 +940,11 @@ init_app (void)
 	GtkCellRenderer *renderer;
 	GSList *i;
 	GtkHBox *vpn_type_hbox1;
+	GtkWidget *toplevel;
 	GDir *dir;
 
-	/* TODO: ensure only one copy of this program is running at any time */
+	if (!vpn_get_clipboard ())
+		return FALSE;
 
 	gconf_client = gconf_client_get_default ();
 	gconf_client_add_dir (gconf_client, NM_GCONF_VPN_CONNECTIONS_PATH,
@@ -931,17 +954,17 @@ init_app (void)
 	xml = glade_xml_new (glade_file, NULL, NULL);
 	g_free (glade_file);
 	if (!xml) {
-		GtkWidget *dialog;
+		GtkWidget *err_dialog;
 
-		dialog = gtk_message_dialog_new (NULL,
+		err_dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
 						 _("Unable to load"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
 		   _("Cannot find some needed resources (the glade file)!"));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+		gtk_dialog_run (GTK_DIALOG (err_dialog));
+		gtk_widget_destroy (err_dialog);
 
 		return FALSE;
 	}
@@ -989,24 +1012,28 @@ init_app (void)
 	gtk_signal_connect (GTK_OBJECT (vpn_export), "clicked", GTK_SIGNAL_FUNC (export_cb), NULL);
 	vpn_delete = glade_xml_get_widget (xml, "delete");
 	gtk_signal_connect (GTK_OBJECT (vpn_delete), "clicked", GTK_SIGNAL_FUNC (delete_cb), NULL);
-	w = glade_xml_get_widget (xml, "close");
-	gtk_signal_connect (GTK_OBJECT (w), "clicked",
-			    GTK_SIGNAL_FUNC (close_cb), NULL);
-	gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
-			    GTK_SIGNAL_FUNC (close_cb), NULL);
-
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (response_cb), NULL);
+	g_signal_connect (dialog, "delete_event",
+			  G_CALLBACK (delete_event_cb), NULL);
 
 	vpn_conn_view = GTK_TREE_VIEW (glade_xml_get_widget (xml, "vpnlist"));
 	vpn_conn_list = gtk_list_store_new (VPNCONN_N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
+	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(vpn_conn_list),
+					VPNCONN_NAME_COLUMN,
+					vpn_list_sorter,
+					NULL,
+					NULL);
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(vpn_conn_list),
+					     VPNCONN_NAME_COLUMN,
+					     GTK_SORT_ASCENDING);
+
 	gtk_signal_connect_after (GTK_OBJECT (vpn_conn_view), "cursor-changed",
 				  GTK_SIGNAL_FUNC (vpn_list_cursor_changed_cb), NULL);
 
-
 	get_all_vpn_connections ();
 
-	
-
 	column = gtk_tree_view_column_new ();
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
@@ -1018,7 +1045,6 @@ init_app (void)
 	gtk_tree_view_set_model (vpn_conn_view, GTK_TREE_MODEL (vpn_conn_list));
 	gtk_tree_view_expand_all (vpn_conn_view);
 
-       
 	gtk_widget_show_all (dialog);
 
 	/* fill in possibly choices in the druid when adding a connection */
@@ -1027,12 +1053,10 @@ init_app (void)
 	for (i = vpn_types; i != NULL; i = g_slist_next (i)) {
 		NetworkManagerVpnUI *vpn_ui = i->data;
 		gtk_combo_box_append_text (vpn_type_combo_box, vpn_ui->get_display_name (vpn_ui));
-
 	}
 	gtk_combo_box_set_active (vpn_type_combo_box, 0);
 	gtk_box_pack_end (GTK_BOX (vpn_type_hbox1), GTK_WIDGET (vpn_type_combo_box), TRUE, TRUE, 0);
 
-
 	/* Druid */
 	druid = GNOME_DRUID (glade_xml_get_widget (xml, "vpn-create-connection-druid"));
 	gtk_signal_connect (GTK_OBJECT (druid), "cancel", GTK_SIGNAL_FUNC (vpn_druid_cancel), NULL);
@@ -1047,15 +1071,15 @@ init_app (void)
 	gtk_signal_connect_after (GTK_OBJECT (w), "prepare", GTK_SIGNAL_FUNC (vpn_druid_vpn_confirm_page_prepare), NULL);
 	gtk_signal_connect_after (GTK_OBJECT (w), "finish", GTK_SIGNAL_FUNC (vpn_druid_vpn_confirm_page_finish), NULL);
 
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (druid));
+	gtk_signal_connect (GTK_OBJECT (toplevel), "delete_event", GTK_SIGNAL_FUNC (vpn_window_close), NULL);
+
 	druid_window = GTK_WINDOW (glade_xml_get_widget (xml, "vpn-create-connection"));
-	/* reuse the widget */
-	gtk_signal_connect (GTK_OBJECT (druid_window), "delete-event", 
-			    GTK_SIGNAL_FUNC (gtk_widget_hide_on_delete), NULL);
+
 	/* make the druid window modal wrt. our main window */
 	gtk_window_set_modal (druid_window, TRUE);
 	gtk_window_set_transient_for (druid_window, GTK_WINDOW (dialog));
 
-
 	/* Edit dialog */
 	edit_dialog = GTK_DIALOG (gtk_dialog_new_with_buttons (_("Edit VPN Connection"),
 							       NULL,
@@ -1065,10 +1089,6 @@ init_app (void)
 							       GTK_STOCK_APPLY,
 							       GTK_RESPONSE_ACCEPT,
 							       NULL));
-	/* reuse the widget */
-	gtk_signal_connect (GTK_OBJECT (edit_dialog), "delete-event", 
-			    GTK_SIGNAL_FUNC (gtk_widget_hide_on_delete), NULL);
-
 
 	/* update "Edit" and "Delete" for current selection */
 	update_edit_del_sensitivity ();
@@ -1076,69 +1096,71 @@ init_app (void)
 	return TRUE;
 }
 
-
 int
 main (int argc, char *argv[])
 {
 	GOptionContext *context;
-	int ret = 1;
+	int ret;
 	gboolean bad_opts;
 	gboolean do_import;
-	GError *error = NULL;
-	static gchar *import_svc = NULL;
-	static gchar *import_file = NULL;
-	static GOptionEntry entries[] = 
-		{
-			{ "import-service", 's', 0,G_OPTION_ARG_STRING, &import_svc, "VPN Service for importing", NULL},
-			{ "import-file", 'f', 0, G_OPTION_ARG_STRING, &import_file, "File to import", NULL},
-			{ NULL }
-		};
+	gchar *import_svc = NULL;
+	gchar *import_file = NULL;
+	GOptionEntry entries[] =  {
+		{ "import-service", 's', 0, G_OPTION_ARG_STRING, &import_svc, "VPN Service for importing", NULL},
+		{ "import-file", 'f', 0, G_OPTION_ARG_FILENAME, &import_file, "File to import", NULL},
+		{ NULL }
+	};
 
-	bindtextdomain (GETTEXT_PACKAGE, NULL);
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
 	context = g_option_context_new ("- NetworkManager VPN properties");
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-	g_option_context_add_group (context, gtk_get_option_group (TRUE));
-	g_option_context_parse (context, &argc, &argv, &error);
+
+#ifdef HAVE_LIBGNOME_2_14
+	gnome_program_init ("nm-vpn-properties", VERSION, LIBGNOMEUI_MODULE, argc, argv,
+			    GNOME_PARAM_GOPTION_CONTEXT, context,
+			    GNOME_PARAM_NONE);
+#else
+	{
+		GError *error = NULL;
+		g_option_context_add_group (context, gtk_get_option_group (TRUE));
+		g_option_context_parse (context, &argc, &argv, &error);
+		g_option_context_free (context);
+	}
+
+	gnome_program_init ("nm-vpn-properties", VERSION, LIBGNOMEUI_MODULE, argc, argv,
+			    GNOME_PARAM_NONE, GNOME_PARAM_NONE);
+#endif
 
 	bad_opts = FALSE;
 	do_import = FALSE;
 	if (import_svc != NULL) {
-		if (import_file != NULL) {
+		if (import_file != NULL)
 			do_import = TRUE;
-		} else {
+		else
 			bad_opts = TRUE;
-		}
-	} else {
-		if (import_file != NULL) {
+	} else if (import_file != NULL)
 			bad_opts = TRUE;
-		}
-	}
 
 	if (bad_opts) {
 		fprintf (stderr, "Have to supply both service and file\n");
+		ret = EXIT_FAILURE;
 		goto out;
 	}
 
-	gnome_program_init (GETTEXT_PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
-			    GNOME_PARAM_NONE);
-
-	glade_gnome_init ();
-
 	if (init_app () == FALSE) {
-		ret = 1;
+		ret = EXIT_FAILURE;
 		goto out;
 	}
 
-	if (do_import) {
-		import_settings (import_svc, import_file);		
-	}
+	if (do_import)
+		import_settings (import_svc, import_file);
 
 	gtk_main ();
 
-	ret = 0;
+	ret = EXIT_SUCCESS;
 
 out:
 	return ret;
