@@ -23,6 +23,10 @@
  * (C) Copyright 2005-2006 SuSE GmbH
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <signal.h>
@@ -94,7 +98,7 @@ void nm_system_device_flush_routes_with_iface (const char *iface)
 	g_return_if_fail (iface != NULL);
 
 	/* Remove routing table entries */
-	buf = g_strdup_printf ("/sbin/ip route flush dev %s", iface);
+	buf = g_strdup_printf (IP_BINARY_PATH " route flush dev %s", iface);
 	nm_spawn_process (buf);
 	g_free (buf);
 }
@@ -131,7 +135,7 @@ void nm_system_device_add_default_route_via_device_with_iface (const char *iface
 	g_return_if_fail (iface != NULL);
 
 	/* Add default gateway */
-	buf = g_strdup_printf ("/sbin/ip route add default dev %s", iface);
+	buf = g_strdup_printf (IP_BINARY_PATH " route add default dev %s", iface);
 	nm_spawn_process (buf);
 	g_free (buf);
 }
@@ -150,7 +154,7 @@ void nm_system_device_add_route_via_device_with_iface (const char *iface, const
 	g_return_if_fail (iface != NULL);
 
 	/* Add default gateway */
-	buf = g_strdup_printf ("/sbin/ip route add %s dev %s", route, iface);
+	buf = g_strdup_printf (IP_BINARY_PATH " route add %s dev %s", route, iface);
 	nm_spawn_process (buf);
 	g_free (buf);
 }
@@ -200,7 +204,7 @@ void nm_system_device_flush_addresses_with_iface (const char *iface)
 	g_return_if_fail (iface != NULL);
 
 	/* Remove all IP addresses for a device */
-	buf = g_strdup_printf ("/sbin/ip addr flush dev %s", iface);
+	buf = g_strdup_printf (IP_BINARY_PATH " addr flush dev %s", iface);
 	nm_spawn_process (buf);
 	g_free (buf);
 }
@@ -214,8 +218,8 @@ void nm_system_device_flush_addresses_with_iface (const char *iface)
  */
 void nm_system_enable_loopback (void)
 {
-	nm_spawn_process ("/sbin/ip link set dev lo up");
-	nm_spawn_process ("/sbin/ip addr add 127.0.0.1/8 brd 127.255.255.255 dev lo scope host label loopback");
+	nm_system_device_set_up_down_with_iface ("lo", TRUE);
+	nm_spawn_process (IP_BINARY_PATH " addr add 127.0.0.1/8 brd 127.255.255.255 dev lo scope host label loopback");
 }
 
 
@@ -228,8 +232,7 @@ void nm_system_enable_loopback (void)
  */
 void nm_system_flush_loopback_routes (void)
 {
-	/* Remove routing table entries for lo */
-	nm_spawn_process ("/sbin/ip route flush dev lo");
+	nm_system_device_flush_routes_with_iface ("lo");
 }
 
 
@@ -241,7 +244,7 @@ void nm_system_flush_loopback_routes (void)
  */
 void nm_system_delete_default_route (void)
 {
-	nm_spawn_process ("/sbin/ip route del default");
+	nm_spawn_process (IP_BINARY_PATH " route del default");
 }
 
 
@@ -253,7 +256,7 @@ void nm_system_delete_default_route (void)
  */
 void nm_system_flush_arp_cache (void)
 {
-	nm_spawn_process ("/sbin/ip neigh flush all");
+	nm_spawn_process (IP_BINARY_PATH " neigh flush all");
 }
 
 
@@ -277,15 +280,8 @@ void nm_system_kill_all_dhcp_daemons (void)
  */
 void nm_system_update_dns (void)
 {
-#ifdef NM_NO_NAMED
-	if (nm_spawn_process (SYSCONFDIR"/init.d/nscd status") != 0)
-		nm_spawn_process (SYSCONFDIR"/init.d/nscd restart");
-
 	nm_info ("Clearing nscd hosts cache.");
 	nm_spawn_process ("/usr/sbin/nscd -i hosts");
-#else
-	nm_spawn_process ("/usr/bin/killall -q nscd");
-#endif
 }
 
 
@@ -329,11 +325,7 @@ void nm_system_device_add_ip6_link_address (NMDevice *dev)
 	struct ether_addr hw_addr;
 	unsigned char eui[8];
 
-	if (nm_device_is_802_3_ethernet (dev))
-		nm_device_802_3_ethernet_get_address (NM_DEVICE_802_3_ETHERNET (dev), &hw_addr);
-	else if (nm_device_is_802_11_wireless (dev))
-		nm_device_802_11_wireless_get_address (NM_DEVICE_802_11_WIRELESS (dev), &hw_addr);
-
+	nm_device_get_hw_address (dev, &hw_addr);
 	memcpy (eui, &(hw_addr.ether_addr_octet), sizeof (hw_addr.ether_addr_octet));
 	memmove (eui+5, eui+3, 3);
 	eui[3] = 0xff;
@@ -341,7 +333,7 @@ void nm_system_device_add_ip6_link_address (NMDevice *dev)
 	eui[0] ^= 2;
 
 	/* Add the default link-local IPv6 address to a device */
-	buf = g_strdup_printf ("/sbin/ip -6 addr add fe80::%x%02x:%x%02x:%x%02x:%x%02x/64 dev %s",
+	buf = g_strdup_printf (IP_BINARY_PATH " -6 addr add fe80::%x%02x:%x%02x:%x%02x:%x%02x/64 dev %s",
 						eui[0], eui[1], eui[2], eui[3], eui[4], eui[5],
 						eui[6], eui[7], nm_device_get_iface (dev));
 	nm_spawn_process (buf);
@@ -354,7 +346,7 @@ typedef struct SuSEDeviceConfigData
 	NMIP4Config *	config;
 	gboolean		use_dhcp;
 	gboolean		system_disabled;
-	unsigned int	mtu;
+	guint32		mtu;
 } SuSEDeviceConfigData;
 
 /*
@@ -462,11 +454,7 @@ void *nm_system_device_get_system_config (NMDevice *dev, NMData *app_data)
 	sys_data = g_malloc0 (sizeof (SuSEDeviceConfigData));
 	sys_data->use_dhcp = TRUE;
 
-	if (nm_device_is_802_3_ethernet (dev))
-		nm_device_802_3_ethernet_get_address (NM_DEVICE_802_3_ETHERNET (dev), &hw_addr);
-	else if (nm_device_is_802_11_wireless (dev))
-		nm_device_802_11_wireless_get_address (NM_DEVICE_802_11_WIRELESS (dev), &hw_addr);
-
+	nm_device_get_hw_address (dev, &hw_addr);
 	sprintf (mac, "%02x:%02x:%02x:%02x:%02x:%02x",
 			hw_addr.ether_addr_octet[0], hw_addr.ether_addr_octet[1],
 			hw_addr.ether_addr_octet[2], hw_addr.ether_addr_octet[3],
@@ -524,12 +512,12 @@ found:
 
 	if ((buf = svGetValue (file, "MTU")))
 	{
-		unsigned long mtu;
+		guint32 mtu;
 
 		errno = 0;
 		mtu = strtoul (buf, NULL, 10);
 		if (!errno && mtu > 500 && mtu < INT_MAX)
-			sys_data->mtu = (unsigned int) mtu;
+			sys_data->mtu = mtu;
 		free (buf);
 	}
 
@@ -616,7 +604,6 @@ found:
 					keyv = g_strsplit (key, "-", 0);
 					real_key = g_strjoinv (NULL, keyv);
 					g_strfreev (keyv);
-					free (key);
 				}
 				security = nm_ap_security_wep_new_from_ap (ap, IW_AUTH_CIPHER_WEP104);
 			}
@@ -664,6 +651,8 @@ found:
 		free (mode);
 		free (buf);
 	}
+	else if (buf)
+		g_free (buf);
 
 	sys_data->config = nm_ip4_config_new ();
 
@@ -722,6 +711,8 @@ found:
 			nm_ip4_config_set_broadcast (sys_data->config, broadcast);
 		}
 
+		nm_ip4_config_set_mtu (sys_data->config, sys_data->mtu);
+
 		buf = NULL;
 		if ((f = fopen (SYSCONFDIR"/sysconfig/network/routes", "r")))
 		{
@@ -811,6 +802,8 @@ void nm_system_device_free_system_config (NMDevice *dev, void *system_config_dat
 
 	if (sys_data->config)
 		nm_ip4_config_unref (sys_data->config);
+
+	g_free (sys_data);
 }
 
 
@@ -846,7 +839,6 @@ gboolean nm_system_device_get_disabled (NMDevice *dev)
 
 	g_return_val_if_fail (dev != NULL, FALSE);
 
-
 	if ((sys_data = nm_device_get_system_config_data (dev)))
 		return sys_data->system_disabled;
 
@@ -1024,13 +1016,15 @@ GSList * nm_system_get_dialup_config (void)
 		NMDialUpConfig *config;
 		shvarFile *modem_file;
 		char *name, *buf, *provider_name;
-		int modem;
+		int type;
 
 		/* we only want modems and isdn */
 		if (g_str_has_prefix (dentry, "ifcfg-modem"))
-			modem = 1;
+			type = NM_DIALUP_TYPE_MODEM;
 		else if (g_str_has_prefix (dentry, "ifcfg-ippp"))
-			modem = 0;
+			type = NM_DIALUP_TYPE_ISDN;
+		else if (g_str_has_prefix (dentry, "ifcfg-dsl"))
+			type = NM_DIALUP_TYPE_DSL;
 		else
 			continue;
 
@@ -1050,15 +1044,21 @@ GSList * nm_system_get_dialup_config (void)
 
 		config = g_malloc (sizeof (NMDialUpConfig));
 		config->data = g_strdup (dentry + 6); /* skip the "ifcfg-" prefix */
-		if (modem)
+		if (type == NM_DIALUP_TYPE_MODEM)
 		{
 			config->name = g_strdup_printf ("%s via modem (%s)", provider_name, (char *) config->data);
 			config->type = NM_DIALUP_TYPE_MODEM;
-		} else
+		}
+		else if (type == NM_DIALUP_TYPE_ISDN)
 		{
 			config->name = g_strdup_printf ("%s via ISDN (%s)", provider_name, (char *) config->data);
 			config->type = NM_DIALUP_TYPE_ISDN;
 		}
+		else if (type == NM_DIALUP_TYPE_DSL)
+		{
+			config->name = g_strdup_printf ("%s via DSL (%s)", provider_name, (char *) config->data);
+			config->type = NM_DIALUP_TYPE_DSL;
+		}
 
 		list = g_slist_append (list, config);
 
@@ -1089,8 +1089,7 @@ void nm_system_activate_nis (NMIP4Config *config)
 {
 	shvarFile *file;
 	const char *nis_domain;
-	char *name, *buf = NULL; 
-	int num_nis_servers = 0; 
+	char *name, *buf;
 	struct in_addr	temp_addr;
 	int i;
 	FILE *ypconf = NULL;
@@ -1117,12 +1116,15 @@ void nm_system_activate_nis (NMIP4Config *config)
 		goto out_close;
 
 	if (!strcmp (buf, "yes")) {
+		int num_nis_servers;
+
 		num_nis_servers = nm_ip4_config_get_num_nis_servers(config);
-		/* write out yp.conf and restart the daemon */
 		if (num_nis_servers > 0)
 		{
 			struct stat sb;
 
+			/* write out yp.conf and restart the daemon */
+
 			ypconf = fopen ("/etc/yp.conf", "w");
 
 			if (ypconf)
@@ -1137,15 +1139,10 @@ void nm_system_activate_nis (NMIP4Config *config)
 			} else
 				nm_warning ("Could not commit NIS changes to /etc/yp.conf.");
 
-			if (stat ("/usr/sbin/rcypbind", &sb) != -1)
-			{
-				nm_info ("Restarting ypbind.");
-				nm_spawn_process ("/usr/sbin/rcypbind restart");
-			}
 			if (stat ("/usr/sbin/rcautofs", &sb) != -1)
 			{
 				nm_info ("Restarting autofs.");
-				nm_spawn_process ("/usr/sbin/rcautofs restart");
+				nm_spawn_process ("/usr/sbin/rcautofs reload");
 			}
 		}
 	}
@@ -1166,20 +1163,9 @@ out_gfree:
  */
 void nm_system_shutdown_nis (void)
 {
-	struct stat sb;
-
-	if (stat ("/usr/sbin/rcypbind", &sb) != -1)
-	{
-		nm_info ("Stopping ypbind.");
-		nm_spawn_process ("/usr/sbin/rcypbind stop");
-	}
-	if (stat ("/usr/sbin/rcautofs", &sb) != -1)
-	{
-		nm_info ("Restarting autofs.");
-		nm_spawn_process ("/usr/sbin/rcautofs restart");
-	}
 }
 
+
 /*
  * nm_system_set_hostname
  *
@@ -1251,12 +1237,12 @@ gboolean nm_system_should_modify_resolv_conf (void)
 	shvarFile *file;
 	gboolean ret = TRUE;
 
-	name = g_strdup_printf (SYSCONFDIR"/sysconfig/network/config");
+	name = g_strdup_printf (SYSCONFDIR"/sysconfig/network/dhcp");
 	file = svNewFile (name);
 	if (!file)
 		goto out_gfree;
 
-	buf = svGetValue (file, "MODIFY_RESOLV_CONF_DYNAMICALLY");
+	buf = svGetValue (file, "DHCLIENT_MODIFY_RESOLV_CONF");
 	if (!buf)
 		goto out_close;
 
@@ -1279,7 +1265,7 @@ out_gfree:
  * Return a user-provided or system-mandated MTU for this device or zero if
  * no such MTU is provided.
  */
-unsigned int nm_system_get_mtu (NMDevice *dev)
+guint32 nm_system_get_mtu (NMDevice *dev)
 {
 	SuSEDeviceConfigData *	sys_data;
 
